import dotenv from 'dotenv';
import { Bus, FisMessage } from "../interface/transport.interface";
import { ConnectionAdapter } from "./connector.base";
import { ReceiverConnectionAdapter as ReceiverConnectionAdapterInterface, Transport, TransportEvent, TransportMessage, TransportService } from "../interface/connector.interface";
import { filter, Observable, Observer, Subscription, takeWhile } from 'rxjs';
import { v4 as uuidv4 } from 'uuid'

dotenv.config();
/* This transport manager will be instantiating the necessary transport to deal with tranmission and receiving from different receivers
So how?: */
export class ReceiverConnectionAdapter extends ConnectionAdapter implements ReceiverConnectionAdapterInterface {
    constructor(adapterId: string, transportType: Transport, adapter: TransportService) {
        super()
        this.connector = adapter
        this.setAdapterProfile(adapterId, transportType)
    }

    setAdapterProfile(id: string, transportType: Transport): void {
        this.connectorProfile = {
            id: id,
            transportType: transportType
        }
    }

    getMessageBus(bus: Bus): Observable<TransportEvent> {
        return new Observable((observable: Observer<TransportEvent>) => {
            if (bus == Bus.GeneralBus) {
                const subscription: Subscription = this.connector.subscribe().pipe(
                    filter((message: TransportEvent) => message.event === 'New Message'),
                    // take message only specific for this adapter. Although that itself wouldn't be necessary, considerng everything goes through transportEvent. I guess it's for better management
                    filter((message: TransportEvent) => (message.data as TransportMessage).target == this.connectorProfile.id),
                    takeWhile((message: TransportEvent) => {
                        const shouldTake = ((message.data as TransportMessage).payload as FisMessage).data !== 'Complete';
                        if (!shouldTake) {
                            observable.complete();  // Ensure the observer is completed
                        }
                        return shouldTake;
                    }),
                ).subscribe((message: TransportEvent) => {
                    observable.next(message);
                });

                // Clean up on unsubscription
                return () => {
                    subscription.unsubscribe();
                };
            }
            if (bus == Bus.ResponseMessageBus) {
                /// logic here
            }
            if (bus == Bus.NotificationMessageBus) {
                /// logic here
            }
            if (bus == Bus.ErrorMessageBus) {
                /// logic here
            }
        });
    }

}


