 import dotenv from 'dotenv';
import { Bus, FisMessage, TransmissionMessage } from "../interface/transport.interface";
import { ConnectionAdapter } from "./connector.base";
import { RequestResponseConnectionAdapter as RequestResponseConnectionAdapterInterface, TransportEvent, TransportMessage, TransportService } from "../interface/connector.interface";
import { TransmitterConnectionAdapter } from './connector.transmitter';
import { ReceiverConnectionAdapter } from './connector.receiver';
import { filter, map, Observable, Observer, Subscription, takeWhile } from 'rxjs';

dotenv.config();
/* This transport manager will be instantiating the necessary transport to deal with tranmission and receiving from different receivers
So how?: */
export class RequestResponseConnectionAdapter extends ConnectionAdapter implements RequestResponseConnectionAdapterInterface {
    private transmitterAdapter!: TransmitterConnectionAdapter
    private receiverAdapter!: ReceiverConnectionAdapter

    constructor( transmitterAdapter: TransmitterConnectionAdapter, receiverAdapter: ReceiverConnectionAdapter) {
        super()
        // logic here
        this.transmitterAdapter = transmitterAdapter
        this.receiverAdapter = receiverAdapter
    }

    // Make use of the adapters ref passed in
    send(message: FisMessage): Observable<FisMessage> {
        return new Observable((response: Observer<FisMessage>) => {
            // logic here
            this.transmitterAdapter.emit(message)
            const subscription: Subscription = this.receiverAdapter.getMessageBus(Bus.GeneralBus).pipe(
                filter((message: TransportEvent) => message.event === 'New Message'),
                // take message only specific for this adapter. Although that itself wouldn't be necessary, considerng everything goes through transportEvent. I guess it's for better management
                filter((message: TransportEvent) => (message.data as TransportMessage).target == this.connectorProfile.id),
                takeWhile((message: TransportEvent) => {
                    const shouldTake = ((message.data as TransportMessage).payload as FisMessage).data !== 'Complete';
                    if (!shouldTake) {
                        response.complete();  // Ensure the observer is completed
                    }
                    return shouldTake;
                }),
                map(message => (message.data as TransportMessage).payload as FisMessage)
            ).subscribe((message: FisMessage) => {
                response.next(message);
            });

            // Clean up on unsubscription
            return () => {
                subscription.unsubscribe();
            };
        })
    }


    /* Extended from both transmitter and receiver */
    getMessageBus(bus: Bus): Observable<TransportEvent> {
        throw new Error('Method not implemented.');
    }

    emit(message: FisMessage): void {
        throw new Error('Method not implemented.');
    }

}

