import { BehaviorSubject, Observable, Subject } from "rxjs"
import { Bus, FisMessage, ReceiverProfile, TransmissionMessage, TransmissionProfile, TransmitterProfile } from "./transport.interface"
import { WrappedMessage } from "../utils/message.ordering"


export type TYPE = {
    adapterProfile: AdapterProfile,
    transmitterProfile: TransmitterProfile,
    receiverProfile: ReceiverProfile,
    requestResponseProfile: TransmissionProfile
}

export interface AdapterProfile {
    id: string,
    transportType: Transport,
}


export interface ConnectionManager {
    getAdapter(clientId: string, transportService: TransportService): AdapterSet
}

export interface ConnectionAdaptorBase {
    connector: TransportService // this one will refer to the actual tranpsort service like websocket and so on
    connectorProfile: AdapterProfile
    connectionStateBus: Subject<ConnectionState>
    adaptorTransmissionRole: AdaptorTransmissionRole
    event: Subject<TransportEvent>

    subscribeConnectionState(): Observable<ConnectionState>
    publishConnectionState(): void
    connect(): void
    disconnect(): void
    getInfo(): AdapterProfile
    setAdapterProfile(id: string, transportType: Transport): void
}


export interface TransmitterConnectionAdapter extends ConnectionAdaptorBase {
    emit(message: FisMessage | WrappedMessage): void
}

export interface ReceiverConnectionAdapter extends ConnectionAdaptorBase {
    getMessageBus(bus: Bus): Observable<any>
}

export interface RequestResponseConnectionAdapter extends TransmitterConnectionAdapter, ReceiverConnectionAdapter {
    send(message: FisMessage): Observable<FisMessage>
}

export type ConnectionState = 'ONLINE' | 'OFFLINE'
export enum AdaptorTransmissionRole {
    Transmitter,
    Receiver,
    RequestResponse
}

export enum Transport {
    Websocket = `Websocket`,
    Grpc = `Grpc`,
    Http = `Http`
}

// TO be used for transmission at the trasport level
export interface TransportMessage {
    id: string,
    dateCreated: Date,
    transport: Transport,
    target?: string,
    payload: any
}

export interface TransportEvent {
    id: string,
    event: Event,
    data: any
}

export type Event = 'Server Started' | 'New Client' | 'Client Disconnected' | 'Client Reconnected' | `Server Disconnected` | 'New Message' | `Notification` | `New Server` | `Server Reconnected` | `New Transport` | 'New Adapter'

export interface TransportService {
    getInfo(): Transport
    emit(message: TransportMessage): void
    subscribe(): Observable<TransportEvent> //all messages and whatever event will go through this, easier to implemnet across different transport protocol
}


export interface Info {
    transport: Transport
}


export interface AdapterSet {
    id: string,
    dateCreated: Date,
    transmitterAdapter: TransmitterConnectionAdapter,
    receiverAdapter: ReceiverConnectionAdapter,
    requestResponsAdapter: RequestResponseConnectionAdapter
}

export interface ClientObject {
    id: string,
    dateCreated: Date,
    connectionState: BehaviorSubject<ConnectionState>
}
