import { Observable, Subject } from "rxjs";
import { AdapterProfile, AdapterSet, AdaptorTransmissionRole, ConnectionManager, RequestResponseConnectionAdapter, TransportEvent, TransportService } from "./connector.interface";
import { MessageTransmissionTransmitter } from "../transmission/msg.transmission.transmitter";
import { MessageTransmissionReceiver } from "../transmission/msg.transmission.receiver";
import { ConnectionAdapter } from "../connector/connector.base";
import { RetransmissionService } from "../utils/retransmission.service";

export interface MessageTransmissionManager {
    subscribe(): Observable<MessageTransmission>
}

export interface MessageTransmission {
    id: string,
    transmitter: MessageTransmissionTransmitter,
    receiver: MessageTransmissionReceiver,
    requestResponse: MessageRequestResponse,
    event: Observable<TransportEvent>
}

export interface MessageTransmissionBase {
    msgRepositoryService: any // like logging service and what not
    transmissionRole: AdaptorTransmissionRole
    adaptorsArray: Array<ConnectionAdapter>

    getInfo(): TransmissionProfile
    setUpAdapter(adapterSet: AdapterSet[]): void
}

export interface MessageReceiver extends MessageTransmissionBase {
    receiverProfile: ReceiverProfile

    getMessageBus(bus: Bus): Observable<any>
    setReceiver(receiverProfile: ReceiverProfile, role: AdaptorTransmissionRole, event: Subject<TransportEvent>): void
}

export interface MessageTransmitter extends MessageTransmissionBase {
    transmitterProfile: TransmitterProfile
    retransmission: RetransmissionService

    setTransmitter(transmitterProfile: TransmitterProfile, role: AdaptorTransmissionRole, event: Subject<TransportEvent>): void
}

export interface MessageRequestResponse extends MessageTransmissionBase {
    incomingMessageBus: Subject<any>
    outgoingMessageBus: Subject<any>
    transmitterInstance: MessageTransmissionTransmitter
    receiverInstance: MessageTransmissionReceiver

    setTransmissionProfile(transmissionInfo: MessageTransmissionTransmitter, receiverInfo: MessageTransmissionReceiver): void
}

export interface FisMessage {
    header: {
        messageID: string,
        messageName: `NotificationMessage` | `ResponseMessage` | `RequestMessage`
    },
    data: any
}

export interface TransmissionProfile {
    id: string,
    name: string,
    dateCreated: Date
}

export interface TransmitterProfile extends TransmissionProfile {
}

export interface ReceiverProfile extends TransmissionProfile {
}

export interface RequestResponseProfile extends TransmissionProfile {

}
export interface TransmissionMessage {
    adapterId: string,
    payload: FisMessage
}

export enum Bus {
    GeneralBus,
    ResponseMessageBus,
    ErrorMessageBus,
    NotificationMessageBus
}

export interface EventMessage {
    clientId: string,
    message: string,
    payload?: any
}