import { filter, map, Observable, Observer, Subject } from "rxjs";
import { Bus, FisMessage, MessageTransmission } from "../interface/transport.interface";
import { v4 as uuidv4 } from 'uuid'
import { MessageTransmissionManager } from "../transmission/msg.transmission.manager";
import { TransportEvent, TransportMessage } from "../interface/connector.interface";
import { WrappedMessage } from "../utils/message.ordering";

class Supervisor {
    private isClient: boolean = true
    private transmissionManager!: MessageTransmissionManager
    private event: Subject<TransportEvent> = new Subject()
    private transmissionSets: MessageTransmission[] = []

    constructor() {
        this.transmissionManager = new MessageTransmissionManager(this.event, this.isClient)

        this.transmissionManager.subscribe().subscribe((transmissionSet: MessageTransmission) => {
            this.transmissionSets.push(transmissionSet)

            this.handleActivity(transmissionSet)
        })
    }

    // only called once for each connected clients.
    private handleActivity(messageTransmission: MessageTransmission): void {
        // start listening to incoming messages from this client
        messageTransmission.receiver.getMessageBus(Bus.GeneralBus).subscribe((event: TransportEvent) => {
            console.log(`General Bus`, event)
        })

        let request: FisMessage = {
            header: {
                messageID: uuidv4(),
                messageName: 'RequestMessage'
            },
            data: 'Data'
        }

        // this.request(request, messageTransmission).subscribe({
        //     next: res => console.log(res),
        //     complete: () => console.log(`Responses Completed for request: ${request.header.messageID}`)
        // })
    }

    private request(request: FisMessage, messageTransmission: MessageTransmission): Observable<any> {
        return new Observable((response: Observer<any>) => {
            messageTransmission.transmitter.emit(request)
            messageTransmission.receiver.getMessageBus(Bus.GeneralBus).pipe(
                filter(event => event.event == 'New Message'),
                filter(event => (((event.data as TransportMessage)?.payload as WrappedMessage)?.payload as FisMessage)?.header.messageID === request.header.messageID),
                map(event => (((event.data as TransportMessage)?.payload as WrappedMessage)?.payload as FisMessage))
            ).subscribe(message => {
                if (message.data == 'Complete') {
                    response.complete()
                } else {
                    response.next(message)
                }
            })
        })
    }

}

let supervisor = new Supervisor()
