import { filter, Observable, Observer, Subject, Subscription } from 'rxjs';
import { AdapterSet, Transport, TransportEvent, TransportMessage } from '../interface/connector.interface';
import { MessageTransmissionBase } from './msg.transmission.base';
import { Bus, EventMessage, MessageReceiver as MessageReceiverInterface, ReceiverProfile } from '../interface/transport.interface'
import { v4 as uuidv4 } from 'uuid'
import { ReceiverConnectionAdapter } from '../connector/connector.receiver';
import { checkMessage, WrappedMessage } from '../utils/message.ordering';

export class MessageTransmissionReceiver extends MessageTransmissionBase implements MessageReceiverInterface {
    private onHoldMessage: Subject<WrappedMessage> = new Subject()
    // private toBePassedOver: Subject<WrappedMessage> = new Subject()
    receiverProfile!: ReceiverProfile;

    constructor(profile: ReceiverProfile, adapterSets: AdapterSet[], event: Observable<TransportEvent>) {
        super(event);

        this.setReceiver(profile)
        this.setUpAdapter(adapterSets)
    }

    setReceiver(receiverProfile: ReceiverProfile): void {
        this.receiverProfile = receiverProfile
    }

    getMessageBus(bus: Bus): Observable<TransportEvent> {
        return new Observable((observable: Observer<TransportEvent>) => {
            // logic here
            if (bus == Bus.GeneralBus) {
                // Need to merge all the adapters into one when the time comes 
                // SAMPLE: This adapterArray.forEach(adapter => { ... })
                const subscription: Subscription = (this.adapterService as ReceiverConnectionAdapter).getMessageBus(Bus.GeneralBus).pipe(
                    filter((event: TransportEvent) => event.event == 'New Message'),
                ).subscribe((event: TransportEvent) => {
                    // console.log(event) // data is transportMessage instead of eventmessage
                    this.onHoldMessage.next(((event.data as TransportMessage).payload as WrappedMessage))
                    checkMessage(((event.data as TransportMessage).payload as WrappedMessage), this.onHoldMessage).then(() => {
                        // only release the message before it exists
                        console.log(`This one passes. Does have previousID. Case for message ordering`) 
                        observable.next(event);
                    }).catch((error) => console.error(error))
                });

                // Clean up on unsubscription
                return () => {
                    subscription.unsubscribe();
                };
            }
        })
    }

    setUpAdapter(adapterSets: AdapterSet[]): void {
        if (adapterSets.length > 0) {
            adapterSets.forEach((adapter: AdapterSet) => {
                this.adaptorsArray.push(adapter.receiverAdapter)
            })
        }
        // for now just hardcode to use 1 adapter type until connection manager is further enhacne to configure adapters on the fly
        this.adapterService = this.adaptorsArray.find(obj => obj.getInfo().transportType === Transport.Websocket)
    }
}
