import { MessageTransmissionBase } from "./msg.transmission.base";
import { FisMessage, MessageRequestResponse as MessageRequestResponseInterface } from '../interface/transport.interface'
import { filter, Observable, Observer, Subject, Subscription, takeWhile } from "rxjs";
import { v4 as uuidv4 } from 'uuid'
import { AdapterSet, Transport, TransportEvent } from "../interface/connector.interface";
import { MessageTransmissionReceiver } from "./msg.transmission.receiver";
import { MessageTransmissionTransmitter } from "./msg.transmission.transmitter";

export class MessageTransmissionRequestResponse extends MessageTransmissionBase implements MessageRequestResponseInterface {
    transmitterInstance!: MessageTransmissionTransmitter;
    receiverInstance!: MessageTransmissionReceiver;
    incomingMessageBus!: Subject<any>;
    outgoingMessageBus!: Subject<any>;

    constructor(transmitterInstance: MessageTransmissionTransmitter, receiverInstance: MessageTransmissionReceiver, event: Observable<TransportEvent>) {
        super(event)
        this.setTransmissionProfile(transmitterInstance, receiverInstance)
    }

    setTransmissionProfile(transmissionInfo: MessageTransmissionTransmitter, receiverInfo: MessageTransmissionReceiver): void {
        this.transmitterInstance = transmissionInfo
        this.receiverInstance = receiverInfo
    }

    // To be Enhanced. This is actually wrong at the moment
    send(message: FisMessage): Observable<FisMessage> {
        return new Observable((response: Observer<FisMessage>) => {
            this.transmitterInstance.emit(message);

            const subscription: Subscription = this.incomingMessageBus
                .pipe(
                    // Filter for matching message IDs
                    filter(incomingMessage => incomingMessage.header.messageID === message.header.messageID),
                    // Complete the observable when the data is 'Complete'
                    takeWhile(incomingMessage => incomingMessage.data !== 'Complete')
                )
                .subscribe({
                    next: incomingMessage => {
                        response.next(incomingMessage);
                        // Complete the observable if data is 'Complete'
                        if (incomingMessage.data === 'Complete') {
                            response.complete();
                            subscription.unsubscribe();
                        }
                    }
                });
        });
    }

    setUpAdapter(adapterSets: AdapterSet[]): void {
        if (adapterSets.length > 0) {
            adapterSets.forEach((adapter: AdapterSet) => {
                this.adaptorsArray.push(adapter.requestResponsAdapter)
            })
        }
        // for now just hardcode to use 1 adapter type until connection manager is further enhacne to configure adapters on the fly
        this.adapterService = this.adaptorsArray.find(obj => obj.getInfo().transportType === Transport.Websocket)
    }
}